@@ -1,5 +1,5 @@
 #!/usr/bin/python -OO
-# Copyright 2008-2012 The SABnzbd-Team <team@sabnzbd.org>
+# Copyright 2008-2015 The SABnzbd-Team <team@sabnzbd.org>
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -24,13 +24,14 @@ import logging
 import re
 
 import sabnzbd
-from sabnzbd.constants import BYTES_FILE_NAME
+from sabnzbd.constants import BYTES_FILE_NAME, BYTES_FILE_NAME_OLD, KIBI
+from sabnzbd.encoding import unicoder
+from math import floor
 import sabnzbd.cfg as cfg
 
-DAY = float(24*60*60)
+DAY = float(24 * 60 * 60)
 WEEK = DAY * 7
 
-#------------------------------------------------------------------------------
 
 def tomorrow(t):
     """ Return timestamp for tomorrow (midnight) """
@@ -97,6 +98,21 @@ def next_month(t):
     return time.mktime(ntime)
 
 
+def fix_keys(data):
+    """ Convert keys of each dictionary in tuple 'data' to unicode """
+    new_data = []
+    if isinstance(data, list):
+        for n in xrange(len(data)):
+            if isinstance(data[n], dict):
+                new = {}
+                for key in data[n]:
+                    new[unicoder(key)] = data[n][key]
+            else:
+                new = data[n]
+            new_data.append(new)
+    return new_data
+
+
 class BPSMeter(object):
     do = None
 
@@ -104,27 +120,30 @@ class BPSMeter(object):
         t = time.time()
         self.start_time = t
         self.log_time = t
+        self.speed_log_time = t
         self.last_update = t
         self.bps = 0.0
+        self.bps_list = []
+        self.bps_list_max = 275
 
         self.day_total = {}
         self.week_total = {}
         self.month_total = {}
         self.grand_total = {}
 
-        self.end_of_day = tomorrow(t)     # Time that current day will end
-        self.end_of_week = next_week(t)   # Time that current day will end
-        self.end_of_month = next_month(t) # Time that current month will end
-        self.q_day = 1                    # Day of quota reset
-        self.q_period = 'm'               # Daily/Weekly/Monthly quota = d/w/m
-        self.quota = self.left = 0.0      # Quota and remaining quota
-        self.have_quota = False           # Flag for quota active
-        self.q_time = 0L            # Next reset time for quota
-        self.q_hour = 0                   # Quota reset hour
-        self.q_minute = 0                 # Quota reset minute
+        self.end_of_day = tomorrow(t)      # Time that current day will end
+        self.end_of_week = next_week(t)    # Time that current day will end
+        self.end_of_month = next_month(t)  # Time that current month will end
+        self.q_day = 1                     # Day of quota reset
+        self.q_period = 'm'                # Daily/Weekly/Monthly quota = d/w/m
+        self.quota = self.left = 0.0       # Quota and remaining quota
+        self.have_quota = False            # Flag for quota active
+        self.q_time = 0L                   # Next reset time for quota
+        self.q_hour = 0                    # Quota reset hour
+        self.q_minute = 0                  # Quota reset minute
+        self.quota_enabled = True          # Scheduled quota enable/disable
         BPSMeter.do = self
 
-
     def save(self):
         """ Save admin to disk """
         if self.grand_total or self.day_total or self.week_total or self.month_total:
@@ -132,39 +151,41 @@ class BPSMeter(object):
                     self.day_total, self.week_total, self.month_total,
                     self.end_of_day, self.end_of_week, self.end_of_month,
                     self.quota, self.left, self.q_time
-                   )
+                    )
             sabnzbd.save_admin(data, BYTES_FILE_NAME)
 
-
     def defaults(self):
-        """ Get the latest data from the database and assign to a fake server
-        """
+        """ Get the latest data from the database and assign to a fake server """
         logging.debug('Setting default BPS meter values')
         import sabnzbd.database
         history_db = sabnzbd.database.get_history_handle()
-        grand, month, week  = history_db.get_history_size()
+        grand, month, week = history_db.get_history_size()
         history_db.close()
         self.grand_total = {}
         self.month_total = {}
         self.week_total = {}
         self.day_total = {}
-        if grand: self.grand_total['x'] = grand
-        if month: self.month_total['x'] = month
-        if week:  self.week_total['x'] = week
+        if grand:
+            self.grand_total['x'] = grand
+        if month:
+            self.month_total['x'] = month
+        if week:
+            self.week_total['x'] = week
         self.quota = self.left = cfg.quota_size.get_float()
 
-
     def read(self):
-        """ Read admin from disk, return True when pause is needed
-        """
+        """ Read admin from disk, return True when pause is needed """
         res = False
-        quota = self.left = cfg.quota_size.get_float() # Quota for this period
+        quota = self.left = cfg.quota_size.get_float()  # Quota for this period
         self.have_quota = bool(cfg.quota_size())
         data = sabnzbd.load_admin(BYTES_FILE_NAME)
+        if not data:
+            data = sabnzbd.load_admin(BYTES_FILE_NAME_OLD)
+            data = fix_keys(data)
         try:
             self.last_update, self.grand_total, \
-            self.day_total, self.week_total, self.month_total, \
-            self.end_of_day, self.end_of_week, self.end_of_month = data[:8]
+                self.day_total, self.week_total, self.month_total, \
+                self.end_of_day, self.end_of_week, self.end_of_month = data[:8]
             if len(data) == 11:
                 self.quota, self.left, self.q_time = data[8:]
                 logging.debug('Read quota q=%s l=%s reset=%s',
@@ -178,17 +199,15 @@ class BPSMeter(object):
             self.defaults()
         # Force update of counters and validate data
         try:
-            for server in self.grand_total:
+            for server in self.grand_total.keys():
                 self.update(server)
         except TypeError:
             self.defaults()
             self.update()
         return res
 
-
     def update(self, server=None, amount=0, testtime=None):
-        """ Update counters for "server" with "amount" bytes
-        """
+        """ Update counters for "server" with "amount" bytes """
         if testtime:
             t = testtime
         else:
@@ -224,18 +243,17 @@ class BPSMeter(object):
             self.grand_total[server] += amount
 
             # Quota check
-            if self.have_quota:
+            if self.have_quota and self.quota_enabled:
                 self.left -= amount
                 if self.left <= 0.0:
                     from sabnzbd.downloader import Downloader
                     if Downloader.do and not Downloader.do.paused:
                         Downloader.do.pause()
-                        logging.warning(Ta('Quota spent, pausing downloading'))
+                        logging.warning(T('Quota spent, pausing downloading'))
 
         # Speedometer
         try:
-            self.bps = (self.bps * (self.last_update - self.start_time)
-                        + amount) / (t - self.start_time)
+            self.bps = (self.bps * (self.last_update - self.start_time) + amount) / (t - self.start_time)
         except:
             self.bps = 0.0
 
@@ -253,6 +271,11 @@ class BPSMeter(object):
             logging.debug("bps: %s", self.bps)
             self.log_time = t
 
+        refresh_rate = cfg.refresh_rate() if cfg.refresh_rate() else 1.0
+        if self.speed_log_time < (t - float(refresh_rate)):
+            self.add_empty_time()
+            self.bps_list.append("%i" % (self.bps / KIBI))
+            self.speed_log_time = t
 
     def reset(self):
         t = time.time()
@@ -261,13 +284,24 @@ class BPSMeter(object):
         self.last_update = t
         self.bps = 0.0
 
+    def add_empty_time(self):
+        refresh_rate = cfg.refresh_rate() if cfg.refresh_rate() else 1.0
+        time_diff = time.time() - self.speed_log_time
+        nr_diffs = int(floor(time_diff / refresh_rate))
+
+        if nr_diffs > 1:
+            self.bps_list.extend(['0.0'] * nr_diffs)
+
+        if len(self.bps_list) > self.bps_list_max:
+            self.bps_list = self.bps_list[len(self.bps_list) - self.bps_list_max:]
+
     def get_sums(self):
         """ return tuple of grand, month, week, day totals """
         return (sum([v for v in self.grand_total.values()]),
                 sum([v for v in self.month_total.values()]),
                 sum([v for v in self.week_total.values()]),
                 sum([v for v in self.day_total.values()])
-               )
+                )
 
     def amounts(self, server):
         """ Return grand, month, week, day totals for specified server """
@@ -278,14 +312,21 @@ class BPSMeter(object):
 
     def clear_server(self, server):
         """ Clean counters for specified server """
-        if server in self.day_total: del self.day_total[server]
-        if server in self.week_total : del self.week_total[server]
-        if server in self.month_total: del self.month_total[server]
-        if server in self.grand_total: del self.grand_total[server]
+        if server in self.day_total:
+            del self.day_total[server]
+        if server in self.week_total:
+            del self.week_total[server]
+        if server in self.month_total:
+            del self.month_total[server]
+        if server in self.grand_total:
+            del self.grand_total[server]
 
     def get_bps(self):
         return self.bps
 
+    def get_bps_list(self):
+        self.add_empty_time()
+        return self.bps_list
 
     def reset_quota(self, force=False):
         """ Check if it's time to reset the quota, optionally resuming
@@ -304,8 +345,7 @@ class BPSMeter(object):
             return True
 
     def next_reset(self, t=None):
-        """ Determine next reset time
-        """
+        """ Determine next reset time """
         t = t or time.time()
         tm = time.localtime(t)
         if self.q_period == 'd':
@@ -315,12 +355,12 @@ class BPSMeter(object):
                 t = time.mktime(nx) + 24 * 3600
                 tm = time.localtime(t)
         elif self.q_period == 'w':
-            if self.q_day < tm.tm_wday+1 or (self.q_day == tm.tm_wday+1 and (tm.tm_hour * 60 + tm.tm_min) >= (self.q_hour * 60 + self.q_minute)):
+            if self.q_day < tm.tm_wday + 1 or (self.q_day == tm.tm_wday + 1 and (tm.tm_hour * 60 + tm.tm_min) >= (self.q_hour * 60 + self.q_minute)):
                 tm = time.localtime(next_week(t))
             dif = abs(self.q_day - tm.tm_wday - 1)
             t = time.mktime(tm) + dif * 24 * 3600
             tm = time.localtime(t)
-        elif self.q_period ==  'm':
+        elif self.q_period == 'm':
             if self.q_day < tm.tm_mday or (self.q_day == tm.tm_mday and (tm.tm_hour * 60 + tm.tm_min) >= (self.q_hour * 60 + self.q_minute)):
                 tm = time.localtime(next_month(t))
             day = min(last_month_day(tm), self.q_day)
@@ -331,10 +371,8 @@ class BPSMeter(object):
         self.q_time = time.mktime(tm)
         logging.debug('Will reset quota at %s', tm)
 
-
     def change_quota(self, allow_resume=True):
-        """ Update quota, potentially pausing downloader
-        """
+        """ Update quota, potentially pausing downloader """
         if not self.have_quota and self.quota < 0.5:
             # Never set, use last period's size
             per = cfg.quota_period()
@@ -360,18 +398,15 @@ class BPSMeter(object):
             self.quota = self.left = 0L
         self.update(0)
         self.next_reset()
-        if self.left > 0.5:
-            from sabnzbd.downloader import Downloader
-            if allow_resume and cfg.quota_resume() and Downloader.do and Downloader.do.paused:
-                Downloader.do.resume()
+        if self.left > 0.5 and allow_resume:
+            self.resume()
 
     # Pattern = <day#> <hh:mm>
     # The <day> and <hh:mm> part can both be optional
     __re_day = re.compile('^\s*(\d+)[^:]*')
     __re_hm = re.compile('(\d+):(\d+)\s*$')
     def get_quota(self):
-        """ If quota active, return check-function, hour, minute
-        """
+        """ If quota active, return check-function, hour, minute """
         if self.have_quota:
             self.q_period = cfg.quota_period()[0].lower()
             self.q_day = 1
@@ -397,10 +432,21 @@ class BPSMeter(object):
         else:
             return None, 0, 0
 
+    def set_status(self, status, action=True):
+        """ Disable/enable quota management """
+        self.quota_enabled = status
+        if action and not status:
+            self.resume()
+
+    def resume(self):
+        """ Resume downloading """
+        from sabnzbd.downloader import Downloader
+        if cfg.quota_resume() and Downloader.do and Downloader.do.paused:
+            Downloader.do.resume()
 
     def midnight(self):
         """ Midnight action: dummy update for all servers """
-        for server in self.day_total:
+        for server in self.day_total.keys():
             self.update(server)
 
 
