@@ -1,5 +1,5 @@
 #!/usr/bin/python -OO
-# Copyright 2008-2012 The SABnzbd-Team <team@sabnzbd.org>
+# Copyright 2008-2015 The SABnzbd-Team <team@sabnzbd.org>
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -22,14 +22,15 @@ sabnzbd.dirscanner - Scanner for Watched Folder
 import os
 import time
 import logging
-import re
 import zipfile
 import gzip
+import bz2
 import threading
 
 import sabnzbd
 from sabnzbd.constants import *
 from sabnzbd.utils.rarfile import is_rarfile, RarFile
+from sabnzbd.newsunpack import is_sevenfile, SevenZip
 import sabnzbd.nzbstuff as nzbstuff
 import sabnzbd.misc as misc
 import sabnzbd.config as config
@@ -37,14 +38,12 @@ import sabnzbd.cfg as cfg
 
 
 def name_to_cat(fname, cat=None):
-    """
-        Retrieve category from file name, but only if "cat" is None.
-    """
+    """ Retrieve category from file name, but only if "cat" is None. """
     if cat is None and fname.startswith('{{'):
         n = fname.find('}}')
         if n > 2:
             cat = fname[2:n].strip()
-            fname = fname[n+2:].strip()
+            fname = fname[n + 2:].strip()
             logging.debug('Job %s has category %s', fname, cat)
 
     return fname, cat
@@ -52,15 +51,19 @@ def name_to_cat(fname, cat=None):
 
 def CompareStat(tup1, tup2):
     """ Test equality of two stat-tuples, content-related parts only """
-    if tup1.st_ino   != tup2.st_ino:   return False
-    if tup1.st_size  != tup2.st_size:  return False
-    if tup1.st_mtime != tup2.st_mtime: return False
-    if tup1.st_ctime != tup2.st_ctime: return False
+    if tup1.st_ino != tup2.st_ino:
+        return False
+    if tup1.st_size != tup2.st_size:
+        return False
+    if tup1.st_mtime != tup2.st_mtime:
+        return False
+    if tup1.st_ctime != tup2.st_ctime:
+        return False
     return True
 
 
 def ProcessArchiveFile(filename, path, pp=None, script=None, cat=None, catdir=None, keep=False,
-                       priority=None, url='', nzbname=None):
+                       priority=None, url='', nzbname=None, password=None, nzo_id=None):
     """ Analyse ZIP file and create job(s).
         Accepts ZIP files with ONLY nzb/nfo/folder files in it.
         returns (status, nzo_ids)
@@ -83,6 +86,11 @@ def ProcessArchiveFile(filename, path, pp=None, script=None, cat=None, catdir=No
             zf = RarFile(path)
         except:
             return -1, []
+    elif is_sevenfile(path):
+        try:
+            zf = SevenZip(path)
+        except:
+            return -1, []
     else:
         return 1, []
 
@@ -108,23 +116,29 @@ def ProcessArchiveFile(filename, path, pp=None, script=None, cat=None, catdir=No
                 except:
                     zf.close()
                     return -1, []
-                name = re.sub(r'\[.*nzbmatrix.com\]', '', name)
                 name = os.path.basename(name)
                 if data:
                     try:
-                        nzo = nzbstuff.NzbObject(name, 0, pp, script, data, cat=cat, url=url,
+                        nzo = nzbstuff.NzbObject(name, pp, script, data, cat=cat, url=url,
                                                  priority=priority, nzbname=nzbname)
+                        if not nzo.password:
+                            nzo.password = password
                     except:
                         nzo = None
                     if nzo:
+                        if nzo_id:
+                            # Re-use existing nzo_id, when a "future" job gets it payload
+                            sabnzbd.nzbqueue.NzbQueue.do.remove(nzo_id, add_to_history=False)
+                            nzo.nzo_id = nzo_id
                         nzo_ids.append(add_nzo(nzo))
                         nzo.update_rating()
         zf.close()
         try:
-            if not keep: os.remove(path)
+            if not keep:
+                os.remove(path)
         except:
-            logging.error(Ta('Error removing %s'), path)
-            logging.info("Traceback: ", exc_info = True)
+            logging.error(T('Error removing %s'), misc.clip_path(path))
+            logging.info("Traceback: ", exc_info=True)
             status = 1
     else:
         zf.close()
@@ -134,9 +148,10 @@ def ProcessArchiveFile(filename, path, pp=None, script=None, cat=None, catdir=No
 
 
 def ProcessSingleFile(filename, path, pp=None, script=None, cat=None, catdir=None, keep=False,
-                      priority=None, nzbname=None, reuse=False, nzo_info=None, dup_check=True, url=''):
-    """ Analyse file and create a job from it
-        Supports NZB, NZB.GZ and GZ.NZB-in-disguise
+                      priority=None, nzbname=None, reuse=False, nzo_info=None, dup_check=True, url='',
+                      password=None, nzo_id=None):
+    """ Analyze file and create a job from it
+        Supports NZB, NZB.BZ2, NZB.GZ and GZ.NZB-in-disguise
         returns (status, nzo_ids)
             status: -2==Error/retry, -1==Error, 0==OK, 1==OK-but-ignorecannot-delete
     """
@@ -155,17 +170,20 @@ def ProcessSingleFile(filename, path, pp=None, script=None, cat=None, catdir=Non
             # gzip file or gzip in disguise
             name = filename.replace('.nzb.gz', '.nzb')
             f = gzip.GzipFile(path, 'rb')
+        elif (b1 == 'B' and b2 == 'Z'):
+            # bz2 file or bz2 in disguise
+            name = filename.replace('.nzb.bz2', '.nzb')
+            f = bz2.BZ2File(path, 'rb')
         else:
             name = filename
             f = open(path, 'rb')
         data = f.read()
         f.close()
     except:
-        logging.warning(Ta('Cannot read %s'), path)
-        logging.info("Traceback: ", exc_info = True)
+        logging.warning(T('Cannot read %s'), misc.clip_path(path))
+        logging.info("Traceback: ", exc_info=True)
         return -2, nzo_ids
 
-
     if name:
         name, cat = name_to_cat(name, catdir)
         # The name is used as the name of the folder, so sanitize it using folder specific santization
@@ -174,10 +192,14 @@ def ProcessSingleFile(filename, path, pp=None, script=None, cat=None, catdir=Non
             nzbname = os.path.split(name)[1]
 
     try:
-        nzo = nzbstuff.NzbObject(name, 0, pp, script, data, cat=cat, priority=priority, nzbname=nzbname,
+        nzo = nzbstuff.NzbObject(name, pp, script, data, cat=cat, priority=priority, nzbname=nzbname,
                                  nzo_info=nzo_info, url=url, reuse=reuse, dup_check=dup_check)
+        if not nzo.password:
+            nzo.password = password
     except TypeError:
         # Duplicate, ignore
+        if nzo_id:
+            sabnzbd.nzbqueue.NzbQueue.do.remove(nzo_id, add_to_history=False)
         nzo = None
     except ValueError:
         # Empty, but correct file
@@ -190,13 +212,18 @@ def ProcessSingleFile(filename, path, pp=None, script=None, cat=None, catdir=Non
             return -1, nzo_ids
 
     if nzo:
-        nzo_ids.append(add_nzo(nzo))
+        if nzo_id:
+            # Re-use existing nzo_id, when a "future" job gets it payload
+            sabnzbd.nzbqueue.NzbQueue.do.remove(nzo_id, add_to_history=False)
+            nzo.nzo_id = nzo_id
+        nzo_ids.append(add_nzo(nzo, quiet=reuse))
         nzo.update_rating()
     try:
-        if not keep: os.remove(path)
+        if not keep:
+            os.remove(path)
     except:
-        logging.error(Ta('Error removing %s'), path)
-        logging.info("Traceback: ", exc_info = True)
+        logging.error(T('Error removing %s'), misc.clip_path(path))
+        logging.info("Traceback: ", exc_info=True)
         return 1, nzo_ids
 
     return 0, nzo_ids
@@ -216,15 +243,13 @@ def CleanList(list, folder, files):
                 del list[path]
 
 
-#------------------------------------------------------------------------------
 class DirScanner(threading.Thread):
+    """ Thread that periodically scans a given directory and picks up any
+        valid NZB, NZB.GZ ZIP-with-only-NZB and even NZB.GZ named as .NZB
+        Candidates which turned out wrong, will be remembered and skipped in
+        subsequent scans, unless changed.
     """
-    Thread that periodically scans a given directoty and picks up any
-    valid NZB, NZB.GZ ZIP-with-only-NZB and even NZB.GZ named as .NZB
-    Candidates which turned out wrong, will be remembered and skipped in
-    subsequent scans, unless changed.
-    """
-    do = None # Access to instance of DirScanner
+    do = None  # Access to instance of DirScanner
 
     def __init__(self):
         threading.Thread.__init__(self)
@@ -237,11 +262,11 @@ class DirScanner(threading.Thread):
                 self.suspected = {}
         except:
             self.ignored = {}   # Will hold all unusable files and the
-                                # successfully processed ones that cannot be deleted
-            self.suspected = {} # Will hold name/attributes of suspected candidates
+            # successfully processed ones that cannot be deleted
+            self.suspected = {}  # Will hold name/attributes of suspected candidates
 
         self.shutdown = False
-        self.error_reported = False # Prevents mulitple reporting of missing watched folder
+        self.error_reported = False  # Prevents mulitple reporting of missing watched folder
         self.dirscan_dir = cfg.dirscan_dir.get_path()
         self.dirscan_speed = cfg.dirscan_speed()
         self.busy = False
@@ -295,7 +320,7 @@ class DirScanner(threading.Thread):
                 files = os.listdir(folder)
             except:
                 if not self.error_reported and not catdir:
-                    logging.error(Ta('Cannot read Watched Folder %s'), folder)
+                    logging.error(T('Cannot read Watched Folder %s'), misc.clip_path(folder))
                     self.error_reported = True
                 files = []
 
@@ -305,7 +330,7 @@ class DirScanner(threading.Thread):
                     continue
 
                 ext = os.path.splitext(path)[1].lower()
-                candidate = ext in ('.nzb', '.zip', '.gz', '.rar')
+                candidate = ext in ('.nzb', '.gz', '.bz2') or ext in VALID_ARCHIVES
                 if candidate:
                     try:
                         stat_tuple = os.stat(path)
@@ -342,8 +367,8 @@ class DirScanner(threading.Thread):
                     if not stable:
                         continue
 
-                    # Handle ZIP files, but only when containing just NZB files
-                    if ext in ('.zip', '.rar') :
+                    # Handle archive files, but only when containing just NZB files
+                    if ext in VALID_ARCHIVES:
                         res, nzo_ids = ProcessArchiveFile(filename, path, catdir=catdir, url=path)
                         if res == -1:
                             self.suspected[path] = stat_tuple
@@ -352,8 +377,8 @@ class DirScanner(threading.Thread):
                         else:
                             self.ignored[path] = 1
 
-                    # Handle .nzb, .nzb.gz or gzip-disguised-as-nzb
-                    elif ext == '.nzb' or filename.lower().endswith('.nzb.gz'):
+                    # Handle .nzb, .nzb.gz or gzip-disguised-as-nzb or .bz2
+                    elif ext == '.nzb' or filename.lower().endswith('.nzb.gz') or filename.lower().endswith('.nzb.bz2'):
                         res, nzo_id = ProcessSingleFile(filename, path, catdir=catdir, url=path)
                         if res < 0:
                             self.suspected[path] = stat_tuple
@@ -378,7 +403,7 @@ class DirScanner(threading.Thread):
                     list = os.listdir(dirscan_dir)
                 except:
                     if not self.error_reported:
-                        logging.error(Ta('Cannot read Watched Folder %s'), dirscan_dir)
+                        logging.error(T('Cannot read Watched Folder %s'), misc.clip_path(dirscan_dir))
                         self.error_reported = True
                     list = []
 
